<?php
/**
 * @Copyright
 *
 * @package	jsvisit_counter for Joomla! 3.x
 * @author     Joachim Schmidt {@link http://www.jschmidt-systemberatung.de/}
 * @version	Version: 2.0.0 - 2-February-2015
 * @link       Project Site {@link http://www.jschmidt-systemberatung.de/}
 *
 * @license GNU/GPL
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * change activity:
 *  02.10-2018: changed to new namespaced Joomla API
 *  03.08.2020: changed server for ip/country detection (2 server)
 *  14.08.2020: processing of ip 127.0.0.1 (localhost) enabled
 *  30.08.2020: added server for ip/country detection (now 3 server)
 *  31.08.2020: added server for ip/country detection (now 4 server)
 *  22.09.2020: use namespaced classes
 *  30.06.2022: change/add code to support jooomla V4
 *  06.08.2022: add support for selection of geolocation server
 */
// No direct access
namespace jsvisit_counter\plugin;
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Version;

class plgAjaxjsvisitHelper
{

	var $db;

	function countVisits ($geolocation_server)
	{
		$debug = false;

		$joomla_version = new Version();

		if ((array_key_exists('REMOTE_ADDR', $_SERVER)))
			$ip = trim($_SERVER['REMOTE_ADDR']);
		else
			$ip = "-1";

		if ($ip == "::1")
			$ip = "127.0.0.1";

		//$ip="93.202.96.212";

		if (version_compare($joomla_version->getShortVersion(), '4.0', '>='))
			$this->db = Factory::getContainer()->get('DatabaseDriver');
		else
			$this->db = Factory::getDbo();

		$today = date("Y-m-d");
		$month = date("m");
		$week = date("W");
		// totals:
		$sql = "UPDATE #__visitors SET count = count + 1, date='" . $today . "' WHERE id = '7'";
		$this->jsvisitDBRequest(true, $sql);

		// monthly stats
		$sql = "select MONTH(date),count from #__visitors where id = 5";
		$result = $this->jsvisitDBRequest(false, $sql);
		if ($month == $result[0])
			$sql = "UPDATE #__visitors SET count = count + 1, date='" . $today . "' WHERE id = '5'";
		else
		{
			$sql = "UPDATE #__visitors SET count = " . $result[1] . ", date='" . $today . "' WHERE id = '6'";
			$this->jsvisitDBRequest(true, $sql);
			$sql = "UPDATE #__visitors SET count = 1, date='" . $today . "' WHERE id = '5'";
		}
		$this->jsvisitDBRequest(true, $sql);

		// this week's stats
		$sql = "select WEEK(date, 3),count from #__visitors where id = 3";
		$result = $this->jsvisitDBRequest(false, $sql);
		if ($week == $result[0])
			$sql = "UPDATE #__visitors SET count = count + 1, date='" . $today . "' WHERE id = '3'";
		else
		{
			$sql = "UPDATE #__visitors SET count = " . $result[1] . ", date='" . $today . "' WHERE id = '4'";
			$this->jsvisitDBRequest(true, $sql);
			$sql = "UPDATE #__visitors SET count = 1, date='" . $today . "' WHERE id = '3'";
		}
		$this->jsvisitDBRequest(true, $sql);

		// today's stats
		$sql = "select date, count from #__visitors where id = 1";
		$result = $this->jsvisitDBRequest(false, $sql);
		if ($today == $result[0])
			$sql = "UPDATE #__visitors SET count = count + 1, date='" . $today . "' WHERE id = '1'";
		else
		{
			$sql = "UPDATE #__visitors SET count = " . $result[1] . ", date='" . $today . "' WHERE id = '2'";
			$this->jsvisitDBRequest(true, $sql);
			$sql = "UPDATE #__visitors SET count = 1, date='" . $today . "' WHERE id = '1'";
		}
		$this->jsvisitDBRequest(true, $sql);

		/* update counts for countries */
		if (in_array(true, $geolocation_server) && $ip != - 1)
		{
			$info = $this->jsvisitgetIpInfo($ip, $geolocation_server);
		}

		if (isset($info['country']['code']))
		{
			$country = strtolower($info['country']['code']);
			$name = $info['country']['name'];
			$sql = "SELECT country FROM #__visitors_country WHERE country ='" . $country . "'";
			$result = $this->jsvisitDBRequest(false, $sql);
			if ($result)
				$sql = "UPDATE #__visitors_country SET count = count + 1 WHERE country='" . $country . "'";
			else
				$sql = "INSERT into #__visitors_country (country, name, count)
                values('" . $country . "', '" . $name . "', '1')";
			$result = $this->jsvisitDBRequest(true, $sql);
		}
		else
		{
			$country = "zz";
			$name = "unknown";
		}

		if ($debug == true)
		{
			$host = gethostbyaddr($ip);
			$date = date("Y-m-d H:i:s");
			if (isset($info['server']['url']))
				$server = $info['server']['url'];
			else
				$server = 'not used';
			$sql = "INSERT into #__visitors_debug (ip, date, country, host, server, name)
            values('" . $ip . "','" . $date . "','" . $country . "', '" . $host . "','" . $server . "','" . $name . "')";
			$result = $this->jsvisitDBRequest(true, $sql);
		}
	}

	function jsvisitgetIpInfo ($ip, $geolocation_server)
	{
		if (filter_var($ip, FILTER_VALIDATE_IP) == false)
			return null;

		if (substr($ip, 0, 3) == "127")
		{
			$lang = Factory::getApplication()->getLanguage();

			$info['country']['code'] = substr($lang->getTag(), 0, 2);
			preg_match('#\((.*?)\)#', $lang->getName(), $name);
			$info['country']['name'] = $name[1];
			$info['server']['url'] = 'localhost';

			return $info;
		}

		$server = array();

		if ($geolocation_server[0])
		{
			$server_props = array(
					'num' => 1,
					'transport' => 'tcp://',
					'url' => "http://www.geoplugin.net/json.gp?ip=" . $ip,
					'host' => 'geoplugin.net',
					'json' => true,
					'port' => 80
			);
			array_push($server, $server_props);
		}

		if ($geolocation_server[1])
		{
			$server_props = array(
					'num' => 2,
					'transport' => 'tcp://',
					'url' => "http://ip-api.com/json/" . $ip,
					'host' => 'ip-api.com',
					'json' => true,
					'port' => 80
			);
			array_push($server, $server_props);
		}

		if ($geolocation_server[2])
		{
			$server_props = array(
					'transport' => 'tcp://',
					'url' => "https://ip2c.org/" . $ip,
					'host' => "ip2c.org",
					'json' => false,
					'port' => 443
			);
			array_push($server, $server_props);
		}

		if ($geolocation_server[3][0])
		{
			$api_key = $geolocation_server[3][1];
			$server_props = array(
					'num' => 3,
					'transport' => 'tcp://',
					'url' => "https://ipapi.co/" . $ip . "/json/?key=" . $api_key,
					'host' => 'ipapi.co',
					'json' => true,
					'port' => 443
			);
			array_push($server, $server_props);
		}

		$i = 0;
		$found = false;
		
		if (count($server) > 1)
		{
			$key = rand(0, count($server)-1);
			while ($key > 0) {
				$temp = array_shift($server);
				$server[] = $temp;
				$key--;
			}
		}

		while ($i < count($server) && $found == false)
		{
			$info = null;

			if (function_exists("curl_init"))
			{
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
				curl_setopt($ch, CURLOPT_PORT, $server[$i]['port']);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
				curl_setopt($ch, CURLOPT_URL, $server[$i]['url']);
				$line = curl_exec($ch);
				if ($line === false)
					echo curl_error($ch);
				curl_close($ch);
			}
			else
			{
				$errno = "";
				$errstr = "";
				$line = "";
				$fp = fsockopen($server[$i]['transport'] . $url['host'], $server[$i]['port'], $errno, $errstr, 30);
				if (! $fp)
				{
					echo "<br /> $errstr";
				}
				else
				{
					$out = "GET " . $url['path'] . " HTTP/1.1\r\n";
					$out .= "Host: " . $url['host'] . "\r\n";
					$out .= "Connection: Close\r\n\r\n";
					fwrite($fp, $out);

					$info = "";
					while (! feof($fp))
					{
						$line .= fgets($fp, 512);
					}

					fclose($fp);
					$line = substr($line, strpos($line, "\r\n\r\n") + 4);
				}
			}

			$tmp = json_decode($line, true);
			$lastserver = $server[$i]['host'];

			if ($server[$i]['json'] == true)
			{
				if ($server[$i]['num'] == 1 && $tmp['geoplugin_status'] == 200)
				{
					$info['country']['code'] = $tmp['geoplugin_countryCode'];
					$info['country']['name'] = $tmp['geoplugin_countryName'];
					$info['server']['url'] = $server[$i]['host'];
				}
				elseif ($server[$i]['num'] == 2 && $tmp['status'] == 'success')
				{
					$info['country']['code'] = $tmp['countryCode'];
					$info['country']['name'] = $tmp['country'];
					$info['server']['url'] = $server[$i]['host'];
				}
				elseif ($server[$i]['num'] == 3 && ! isset($tmp['error']))
				{
					$info['country']['code'] = $tmp['country_code'];
					$info['country']['name'] = $tmp['country_name'];
					$info['server']['url'] = $server[$i]['host'];
				}
				elseif (isset($tmp['error']))
				{
					$info['country']['code'] = 'zz';
					$info['country']['name'] = $tmp['message'];
					$info['server']['url'] = $server[$i]['host'];
			    }
			}
			else
			{
				$tmp = explode(";", $line);
				if (count($tmp) > 3 && $tmp[0] == "1")
				{
					$info['country']['code'] = $tmp[1];
					$info['country']['name'] = $tmp[3];
					$info['server']['url'] = $server[$i]['host'];
				}
			}

			if (isset($info['country']['code']))
			{
				$found = true;
				$i = count($server) + 1;
				return $info;
			}
			

			$i ++;
		}

		$info['country']['code'] = "zz";
		$info['country']['name'] = "unknown";
		$info['server']['url'] = $lastserver;

		return $info;
	}

	function jsvisitDBRequest ($update, $sql)
	{
		if ($update == true)
		{
			$result = $this->db->setQuery($sql);
			$this->db->execute();
			return $result;
		}
		else
		{
			$result = $this->db->setQuery($sql);
			$row = $this->db->loadRow();
			return $row;
		}
		return - 1;
	}
}